#ifndef RCT_H__
#define RCT_H__

#include <stdint.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

// RCT 8-bit
void linelet_rct_forward_argb8888(int16_t * const *output, size_t stride,
      const uint32_t *input, size_t width, size_t height, size_t in_stride);

void linelet_rct_inverse_argb8888(uint32_t *output, size_t out_stride,
      const int16_t * const *input, size_t width, size_t height, size_t stride);

void linelet_rct_forward_gbrp8(int16_t * const *output, size_t stride,
      const uint8_t * const *input, size_t width, size_t height, const size_t *in_stride);

void linelet_rct_inverse_gbrp8(uint8_t * const *output, const size_t *stride,
      const int16_t * const *input, size_t width, size_t height, size_t in_stride);

// RCT 10-bit
void linelet_rct_forward_a2bgr10_yuv10(int16_t * const *output, size_t stride,
      const uint32_t *input, size_t width, size_t height, size_t in_stride);

void linelet_rct_inverse_yuv10_a2bgr10(uint32_t *output, size_t out_stride,
      const int16_t * const *input, size_t width, size_t height, size_t stride);

void linelet_rct_forward_gbrp10(int16_t * const *output, size_t stride,
      const uint16_t * const *input, size_t width, size_t height, const size_t *in_stride);

void linelet_rct_inverse_gbrp10(uint16_t * const *output, const size_t *stride,
      const int16_t * const *input, size_t width, size_t height, size_t in_stride);

// BT709/BT601 8-bit
void linelet_bt_yuv8_forward_dc_shift(int16_t * const *output, const size_t *stride,
      const uint8_t * const *input, size_t *width, size_t height, const size_t *in_stride);

void linelet_bt_yuv8_inverse_dc_shift(uint8_t * const *output, const size_t *stride,
      const int16_t * const *input, size_t *width, size_t height, const size_t *in_stride);

// BT709/BT601 10-bit
void linelet_bt_yuv10_forward_dc_shift(int16_t * const *output, const size_t *stride,
      const uint16_t * const *input, size_t *width, size_t height, const size_t *in_stride);

void linelet_bt_yuv10_inverse_dc_shift(uint16_t * const *output, const size_t *stride,
      const int16_t * const *input, size_t *width, size_t height, const size_t *in_stride);

// BT709/BT601 12-bit
void linelet_bt_yuv12_forward_dc_shift(int16_t * const *output, const size_t *stride,
      const uint16_t * const *input, size_t *width, size_t height, const size_t *in_stride);

void linelet_bt_yuv12_inverse_dc_shift(uint16_t * const *output, const size_t *stride,
      const int16_t * const *input, size_t *width, size_t height, const size_t *in_stride);

#ifdef __cplusplus
extern "C" {
#endif

#endif

