#ifndef IMAGE_H__
#define IMAGE_H__

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#else
#include <stdbool.h>
#endif

struct linelet_yuv_image
{
   struct
   {
      int16_t *data;
      size_t stride;
      unsigned sub;
   } luma, cb, cr;

   size_t width;
   size_t height;

   int16_t *components[3];
   size_t strides[3];

   unsigned bits;
};

struct linelet_rgb_image
{
   uint32_t *rgb;
   size_t width;
   size_t height;
   size_t stride;
   size_t size;

   unsigned bits;
};

bool linelet_rgb_image_load(const char *path, struct linelet_rgb_image *img);
bool linelet_rgb_image_save(const char *path, const struct linelet_rgb_image *img);
void linelet_rgb_image_free(struct linelet_rgb_image *img);

bool linelet_yuv_image_load(const char *path, struct linelet_yuv_image *img);
bool linelet_yuv_image_save(const char *path, const struct linelet_yuv_image *img);
void linelet_yuv_image_free(struct linelet_yuv_image *img);

bool linelet_yuv_image_alloc(struct linelet_yuv_image *img, size_t width, size_t height,
      unsigned sub_luma, unsigned sub_cb, unsigned sub_cr);
bool linelet_rgb_image_alloc(struct linelet_rgb_image *img, size_t width, size_t height);

struct linelet_yuv_psnr
{
   struct
   {
      double luma, cb, cr; 
   } mse;
   double luma, cb, cr;
};

bool linelet_yuv_image_psnr(struct linelet_yuv_psnr *psnr, const struct linelet_yuv_image *a, const struct linelet_yuv_image *b, unsigned bits);

struct linelet_rgb_psnr
{
   struct
   {
      double r, g, b; 
   } mse;
   double r, g, b;
};

bool linelet_rgb_image_psnr(struct linelet_rgb_psnr *psnr, const struct linelet_rgb_image *a, const struct linelet_rgb_image *b, unsigned bits);

bool linelet_rgb_image_compute_error_image(struct linelet_rgb_image *out,
      const struct linelet_rgb_image *ref, const struct linelet_rgb_image *res);

double linelet_image_component_mse_16(const uint16_t *a, const uint16_t *b, unsigned samples);
double linelet_image_component_mse_8(const uint8_t *a, const uint8_t *b, unsigned samples);

double linelet_image_component_ssim_16(const uint16_t *a, const uint16_t *b, unsigned width, unsigned height, unsigned bits);
double linelet_image_component_ssim_8(const uint8_t *a, const uint8_t *b, unsigned width, unsigned height);

#ifdef __cplusplus
}
#endif

#endif

