#ifndef LINELET_LOG_H__
#define LINELET_LOG_H__

#include <limits.h>
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

enum linelet_log_level
{
   LINELET_LOG_DEBUG = 0,
   LINELET_LOG_INFO,
   LINELET_LOG_WARN,
   LINELET_LOG_ERROR,

   LINELET_LOG_DUMMY = INT_MAX
};

typedef void (*linelet_vlog_callback_t)(void *userdata, enum linelet_log_level level, const char *fmt, va_list va);

void linelet_log_set_callback(linelet_vlog_callback_t cb, void *userdata);
void linelet_log_set_log_level(enum linelet_log_level level);

#ifdef __GNUC__
#define LINELET_FORMAT_PRINTF __attribute__((format(printf, 2, 3)))
#define LINELET_FORMAT_VPRINTF __attribute__((format(printf, 2, 0)))
#else
#define LINELET_FORMAT_PRINTF
#define LINELET_FORMAT_VPRINTF
#endif

void linelet_log_printf(enum linelet_log_level level, const char *fmt, ...) LINELET_FORMAT_PRINTF;
void linelet_log_vprintf(enum linelet_log_level level, const char *fmt, va_list va) LINELET_FORMAT_VPRINTF;

#ifdef __cplusplus
}
#endif
#endif

