#ifndef LINELET_SIMD_H__
#define LINELET_SIMD_H__

#include <stdint.h>
#include "rate_allocator.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*linelet_emms_t)(void);

typedef void (*linelet_rct_forward_argb8_yuv8_line_t)(int16_t *l, int16_t *cb, int16_t *cr, const uint32_t *src, unsigned samples);
typedef void (*linelet_rct_forward_a2bgr10_yuv10_line_t)(int16_t *l, int16_t *cb, int16_t *cr, const uint32_t *src, unsigned samples);
typedef void (*linelet_rct_inverse_yuv8_argb8_line_t)(uint32_t *dst, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples);
typedef void (*linelet_rct_inverse_yuv10_a2bgr10_line_t)(uint32_t *dst, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples);

typedef void (*linelet_rct_forward_gbrp8_line_t)(int16_t *l, int16_t *cb, int16_t *cr, const uint8_t *g, const uint8_t *b, const uint8_t *r, unsigned samples);
typedef void (*linelet_rct_inverse_gbrp8_line_t)(uint8_t *g, uint8_t *b, uint8_t *r, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples);
typedef void (*linelet_rct_forward_gbrp10_line_t)(int16_t *l, int16_t *cb, int16_t *cr, const uint16_t *g, const uint16_t *b, const uint16_t *r, unsigned samples);
typedef void (*linelet_rct_inverse_gbrp10_line_t)(uint16_t *g, uint16_t *b, uint16_t *r, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples);

typedef void (*linelet_bt_yuv8_forward_line_t)(int16_t *dst, const uint8_t *src, unsigned samples);
typedef void (*linelet_bt_yuv8_inverse_line_t)(uint8_t *dst, const int16_t *src, unsigned samples);
typedef void (*linelet_bt_yuv10_forward_line_t)(int16_t *dst, const uint16_t *src, unsigned samples);
typedef void (*linelet_bt_yuv10_inverse_line_t)(uint16_t *dst, const int16_t *src, unsigned samples);
typedef void (*linelet_bt_yuv12_forward_line_t)(int16_t *dst, const uint16_t *src, unsigned samples);
typedef void (*linelet_bt_yuv12_inverse_line_t)(uint16_t *dst, const int16_t *src, unsigned samples);

typedef void (*linelet_haar_line_forward_t)(int16_t *a, int16_t *b, unsigned samples);
typedef void (*linelet_haar_line_inverse_t)(int16_t *a, int16_t *b, unsigned samples);

typedef void (*linelet_legall_vert_line_forward_top_t)(
      int16_t *lp0,
      int16_t *hp0,
      int16_t *lp1,
      unsigned samples);

typedef void (*linelet_legall_vert_line_forward_t)(
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      int16_t *lp2,
      unsigned samples);

typedef void (*linelet_legall_vert_line_forward_bottom_t)(
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      unsigned samples);

typedef void (*linelet_legall_vert_line_inverse_top_t)(
      int16_t *lp0,
      int16_t *hp0,
      unsigned samples);

typedef void (*linelet_legall_vert_line_inverse_t)(
      int16_t *lp0,
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      unsigned samples);

typedef void (*linelet_legall_vert_line_inverse_bottom_t)(
      int16_t *lp0,
      int16_t *hp0,
      unsigned samples);

typedef void (*linelet_legall_line_forward_t)(int16_t *lp, int16_t *hp, const int16_t *src, unsigned samples);
typedef void (*linelet_legall_line_inverse_t)(int16_t *dst, const int16_t *lp, const int16_t *hp, unsigned samples);

typedef void (*linelet_quantize_line_t)(int16_t *data, unsigned samples, unsigned q, int16_t adder);
typedef void (*linelet_dequantize_line_t)(int16_t *data, unsigned samples, unsigned q);

typedef void (*linelet_range_min_max_line_8_t)(struct linelet_precinct_range_level *ranges,
      const int16_t *src, unsigned precincts, unsigned step);
typedef void (*linelet_range_min_max_line_4_t)(struct linelet_precinct_range_level *ranges,
      const int16_t *src, unsigned precincts);

typedef uint8_t *(*linelet_pack16_t)(uint8_t *output, const int16_t *input, unsigned samples);
typedef uint8_t *(*linelet_pack8_t)(uint8_t *output, const int16_t *input);
typedef uint8_t *(*linelet_pack4_t)(uint8_t *output, const int16_t *input);
typedef const uint8_t *(*linelet_unpack8_t)(int16_t *output, const uint8_t *input, unsigned samples, const uint64_t *lut);
typedef const uint8_t *(*linelet_unpack4_t)(int16_t *output, const uint8_t *input, const uint64_t *lut);

typedef void (*linelet_prefilter_line2_t)(int16_t *l0, int16_t *l1, unsigned samples);
typedef void (*linelet_postfilter_line2_t)(int16_t *l0, int16_t *l1, unsigned samples);
typedef void (*linelet_prefilter_line4_t)(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, unsigned samples);
typedef void (*linelet_postfilter_line4_t)(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, unsigned samples);
typedef void (*linelet_prefilter_line8_t)(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, int16_t *l4, int16_t *l5, int16_t *l6, int16_t *l7, unsigned samples);
typedef void (*linelet_postfilter_line8_t)(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, int16_t *l4, int16_t *l5, int16_t *l6, int16_t *l7, unsigned samples);

struct linelet_simd_funcs
{
   linelet_prefilter_line2_t prefilter_line2;
   linelet_postfilter_line2_t postfilter_line2;
   linelet_prefilter_line4_t prefilter_line4;
   linelet_postfilter_line4_t postfilter_line4;
   linelet_prefilter_line8_t prefilter_line8;
   linelet_postfilter_line8_t postfilter_line8;

   linelet_rct_forward_argb8_yuv8_line_t rct_forward_argb8_yuv8_line;
   linelet_rct_forward_a2bgr10_yuv10_line_t rct_forward_a2bgr10_yuv10_line;
   linelet_rct_inverse_yuv8_argb8_line_t rct_inverse_yuv8_argb8_line;
   linelet_rct_inverse_yuv10_a2bgr10_line_t rct_inverse_yuv10_a2bgr10_line;

   linelet_rct_forward_gbrp8_line_t rct_forward_gbrp8_line;
   linelet_rct_inverse_gbrp8_line_t rct_inverse_gbrp8_line;
   linelet_rct_forward_gbrp10_line_t rct_forward_gbrp10_line;
   linelet_rct_inverse_gbrp10_line_t rct_inverse_gbrp10_line;

   linelet_bt_yuv8_forward_line_t bt_yuv8_forward_line;
   linelet_bt_yuv8_inverse_line_t bt_yuv8_inverse_line;
   linelet_bt_yuv10_forward_line_t bt_yuv10_forward_line;
   linelet_bt_yuv10_inverse_line_t bt_yuv10_inverse_line;
   linelet_bt_yuv12_forward_line_t bt_yuv12_forward_line;
   linelet_bt_yuv12_inverse_line_t bt_yuv12_inverse_line;

   linelet_haar_line_forward_t haar_line_forward;
   linelet_haar_line_inverse_t haar_line_inverse;

   linelet_legall_line_forward_t legall_line_forward;
   linelet_legall_line_inverse_t legall_line_inverse;

   linelet_legall_vert_line_forward_top_t legall_vert_line_forward_top;
   linelet_legall_vert_line_forward_t legall_vert_line_forward;
   linelet_legall_vert_line_forward_bottom_t legall_vert_line_forward_bottom;

   linelet_legall_vert_line_inverse_top_t legall_vert_line_inverse_top;
   linelet_legall_vert_line_inverse_t legall_vert_line_inverse;
   linelet_legall_vert_line_inverse_bottom_t legall_vert_line_inverse_bottom;

   linelet_quantize_line_t quantize_line;
   linelet_dequantize_line_t dequantize_line;

   linelet_range_min_max_line_8_t range_min_max_line_8;
   linelet_range_min_max_line_4_t range_min_max_line_4;

   linelet_pack16_t pack16[16];
   linelet_pack8_t pack8[16];
   linelet_pack4_t pack4[16];
   linelet_unpack8_t unpack8[16];
   linelet_unpack4_t unpack4[16];

   linelet_emms_t emms;
};

extern struct linelet_simd_funcs linelet_simd;

void linelet_simd_init_c(void); // Baseline init.
void linelet_simd_init_x86(void);

#ifdef __cplusplus
}
#endif
#endif
