#include "convert.h"
#include "timer.h"

void linelet_rct_forward_argb8_yuv8_line_c(int16_t *l, int16_t *cb, int16_t *cr, const uint32_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      uint32_t col = src[x];
      int16_t r = (col >> 16) & 0xff;
      int16_t g = (col >>  8) & 0xff;
      int16_t b = (col >>  0) & 0xff;

      l[x] = ((r + (g << 1) + b) >> 2) - 128;
      cb[x] = b - g;
      cr[x] = r - g;
   }
}

void linelet_rct_forward_gbrp8_line_c(int16_t *l, int16_t *cb, int16_t *cr,
      const uint8_t *G, const uint8_t *B, const uint8_t *R, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t g = G[x];
      int16_t b = B[x];
      int16_t r = R[x];

      l[x] = ((r + (g << 1) + b) >> 2) - 128;
      cb[x] = b - g;
      cr[x] = r - g;
   }
}

static inline int16_t clamp(int16_t v, int16_t min, int16_t max)
{
   if (v < min)
      return min;
   else if (v > max)
      return max;
   else
      return v;
}

void linelet_rct_inverse_yuv8_argb8_line_c(uint32_t *dst, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l_samp  = l[x];
      int16_t cb_samp = cb[x];
      int16_t cr_samp = cr[x];

      int16_t g = l_samp - ((cb_samp + cr_samp) >> 2);
      int16_t r = cr_samp + g;
      int16_t b = cb_samp + g;

      g += 128;
      r += 128;
      b += 128;

      g = clamp(g, 0, 255);
      r = clamp(r, 0, 255);
      b = clamp(b, 0, 255);

      dst[x] = (r << 16) | (g << 8) | (b << 0);
   }
}

void linelet_rct_inverse_gbrp8_line_c(uint8_t *G, uint8_t *B, uint8_t *R,
      const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l_samp  = l[x];
      int16_t cb_samp = cb[x];
      int16_t cr_samp = cr[x];

      int16_t g = l_samp - ((cb_samp + cr_samp) >> 2);
      int16_t r = cr_samp + g;
      int16_t b = cb_samp + g;

      g += 128;
      r += 128;
      b += 128;

      G[x] = clamp(g, 0, 255);
      B[x] = clamp(b, 0, 255);
      R[x] = clamp(r, 0, 255);
   }
}

void linelet_rct_forward_a2bgr10_yuv10_line_c(int16_t *l, int16_t *cb, int16_t *cr, const uint32_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      uint32_t col = src[x];
      int16_t r = (col >>  0) & 0x3ff;
      int16_t g = (col >> 10) & 0x3ff;
      int16_t b = (col >> 20) & 0x3ff;

      l[x] = ((r + (g << 1) + b) >> 2) - 512;
      cb[x] = b - g;
      cr[x] = r - g;
   }
}

void linelet_rct_forward_gbrp10_line_c(int16_t *l, int16_t *cb, int16_t *cr,
      const uint16_t *G, const uint16_t *B, const uint16_t *R, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t g = G[x];
      int16_t b = B[x];
      int16_t r = R[x];

      l[x] = ((r + (g << 1) + b) >> 2) - 512;
      cb[x] = b - g;
      cr[x] = r - g;
   }
}

void linelet_rct_inverse_yuv10_a2bgr10_line_c(uint32_t *output, const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l_samp  = l[x];
      int16_t cb_samp = cb[x];
      int16_t cr_samp = cr[x];

      int16_t g = l_samp - ((cb_samp + cr_samp) >> 2);
      int16_t r = cr_samp + g;
      int16_t b = cb_samp + g;

      g += 512;
      r += 512;
      b += 512;

      uint32_t r_out = clamp(r, 0, 1023);
      uint32_t g_out = clamp(g, 0, 1023);
      uint32_t b_out = clamp(b, 0, 1023);

      output[x] = (r_out << 0) | (g_out << 10) | (b_out << 20);
   }
}

void linelet_rct_inverse_gbrp10_line_c(uint16_t *G, uint16_t *B, uint16_t *R,
      const int16_t *l, const int16_t *cb, const int16_t *cr, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l_samp  = l[x];
      int16_t cb_samp = cb[x];
      int16_t cr_samp = cr[x];

      int16_t g = l_samp - ((cb_samp + cr_samp) >> 2);
      int16_t r = cr_samp + g;
      int16_t b = cb_samp + g;

      g += 512;
      r += 512;
      b += 512;

      G[x] = clamp(g, 0, 1023);
      B[x] = clamp(b, 0, 1023);
      R[x] = clamp(r, 0, 1023);
   }
}

void linelet_bt_yuv8_forward_line_c(int16_t *dst, const uint8_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = src[x] - 128;
}

void linelet_bt_yuv8_inverse_line_c(uint8_t *dst, const int16_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = clamp(src[x] + 128, 0, 255); // FIXME: Determine if this should be TV range or PC range.
}

void linelet_bt_yuv10_forward_line_c(int16_t *dst, const uint16_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = src[x] - 512;
}

void linelet_bt_yuv10_inverse_line_c(uint16_t *dst, const int16_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = clamp(src[x] + 512, 0, 1023); // FIXME: Determine if this should be TV range or PC range.
}

void linelet_bt_yuv12_forward_line_c(int16_t *dst, const uint16_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = src[x] - 2048;
}

void linelet_bt_yuv12_inverse_line_c(uint16_t *dst, const int16_t *src, unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
      dst[x] = clamp(src[x] + 2048, 0, 4095); // FIXME: Determine if this should be TV range or PC range.
}

