#include "legall.h"
#include "timer.h"

void linelet_legall_line_forward_c(int16_t *lp, int16_t *hp, const int16_t *in, unsigned samples)
{
   int16_t last_hp = 0;
   for (unsigned x = 0, target = 0; x < samples; x += 2, target++)
   {
      hp[target] = in[x + 1] - ((in[x] + in[x + 2]) >> 1);
      lp[target] = in[x] + ((last_hp + hp[target] + 2) >> 2);
      last_hp = hp[target];
   }
}

void linelet_legall_line_inverse_c(int16_t *out, const int16_t *lp, const int16_t *hp, unsigned samples)
{
   for (int x = 0, target = 0; x < (int)samples; x += 2, target++)
   {
      out[x + 0] = lp[target] - ((hp[target] + hp[target - 1] + 2) >> 2);
      out[x - 1] = hp[target - 1] + ((out[x - 2] + out[x]) >> 1);
   }
}

void linelet_legall_vert_line_forward_top_c(
      int16_t *lp0,
      int16_t *hp0,
      int16_t *lp1,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];

      h0 -= (l0 + l1) >> 1;
      l0 += ((h0 << 1) + 2) >> 2;

      lp0[x] = l0;
      hp0[x] = h0;
   }
}

void linelet_legall_vert_line_forward_c(
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      int16_t *lp2,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];
      int16_t l2 = lp2[x];

      h1 -= (l1 + l2) >> 1;
      l1 += (h0 + h1 + 2) >> 2;

      lp1[x] = l1;
      hp1[x] = h1;
   }
}

void linelet_legall_vert_line_forward_bottom_c(
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];

      h1 -= l1;
      l1 += (h0 + h1 + 2) >> 2;

      lp1[x] = l1;
      hp1[x] = h1;
   }
}

void linelet_legall_vert_line_inverse_top_c(
      int16_t *lp0,
      int16_t *hp0,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];

      l0 -= (h0 + 1) >> 1;
      lp0[x] = l0;
   }
}

void linelet_legall_vert_line_inverse_c(
      int16_t *lp0,
      int16_t *hp0,
      int16_t *lp1,
      int16_t *hp1,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];

      l1 -= (h0 + h1 + 2) >> 2;
      h0 += (l0 + l1) >> 1;

      hp0[x] = h0;
      lp1[x] = l1;
   }
}

void linelet_legall_vert_line_inverse_bottom_c(
      int16_t *lp0,
      int16_t *hp0,
      unsigned samples)
{
   for (unsigned x = 0; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];

      h0 += l0;
      hp0[x] = h0;
   }
}

