#include "pack.h"
#include "memory.h"
#include <stdint.h>
#include <stdio.h>

#define PACK_16(bits) \
uint8_t *linelet_pack_line16_##bits##_c(uint8_t *output, const int16_t *input, unsigned samples) \
{ \
   for (unsigned i = 0; i < samples; i += 8, input += 8, output += bits) \
   { \
      for (unsigned b = 0; b < bits; b++) \
      { \
         output[b] = (\
               (((input[0] >> (bits - 1 - b)) & 1) << 0) | \
               (((input[1] >> (bits - 1 - b)) & 1) << 1) | \
               (((input[2] >> (bits - 1 - b)) & 1) << 2) | \
               (((input[3] >> (bits - 1 - b)) & 1) << 3) | \
               (((input[4] >> (bits - 1 - b)) & 1) << 4) | \
               (((input[5] >> (bits - 1 - b)) & 1) << 5) | \
               (((input[6] >> (bits - 1 - b)) & 1) << 6) | \
               (((input[7] >> (bits - 1 - b)) & 1) << 7)); \
      } \
   } \
   return output; \
}

#define PACK_8(bits) \
uint8_t *linelet_pack_line8_##bits##_c(uint8_t *output, const int16_t *input) \
{ \
   for (unsigned b = 0; b < bits; b++) \
   { \
      output[b] = (\
            (((input[0] >> (bits - 1 - b)) & 1) << 0) | \
            (((input[1] >> (bits - 1 - b)) & 1) << 1) | \
            (((input[2] >> (bits - 1 - b)) & 1) << 2) | \
            (((input[3] >> (bits - 1 - b)) & 1) << 3) | \
            (((input[4] >> (bits - 1 - b)) & 1) << 4) | \
            (((input[5] >> (bits - 1 - b)) & 1) << 5) | \
            (((input[6] >> (bits - 1 - b)) & 1) << 6) | \
            (((input[7] >> (bits - 1 - b)) & 1) << 7)); \
   } \
   return output + bits; \
}

#define PACK_4(bits) \
uint8_t *linelet_pack_line4_##bits##_c(uint8_t *output, const int16_t *input) \
{ \
   uint64_t res = 0; \
   for (unsigned b = 0; b < bits; b++) \
   { \
      res <<= 4; \
      res |= ( \
       (((input[0] >> b) & 1) << 0) | \
       (((input[1] >> b) & 1) << 1) | \
       (((input[2] >> b) & 1) << 2) | \
       (((input[3] >> b) & 1) << 3)); \
   } \
   unsigned bytes = (bits + 1) >> 1; \
   for (unsigned b = 0; b < bytes; b++) \
   { \
      *output++ = res; \
      res >>= 8; \
   } \
   return output; \
}

#define UNPACK_8_8(bits) \
const uint8_t *linelet_unpack_line8_##bits##_c(int16_t *output, const uint8_t *input, unsigned samples, const uint64_t *lut) \
{ \
   for (unsigned i = 0; i < samples; i += 8, input += bits, output += 8) \
   { \
      uint64_t res = 0; \
      for (unsigned b = 0; b < bits; b++) \
         res |= lut[input[b]] << (bits - 1 - b); \
\
      for (unsigned s = 0; s < 8; s++) \
      { \
         uint8_t v = res; \
         res >>= 8; \
         output[s] = ((int16_t)(v << (16 - bits))) >> (16 - bits); \
      } \
   } \
   return input; \
}

#define UNPACK_8_16(bits) \
const uint8_t *linelet_unpack_line8_##bits##_c(int16_t *output, const uint8_t *input, unsigned samples, const uint64_t *lut) \
{ \
   for (unsigned i = 0; i < samples; i += 8, input += bits, output += 8) \
   { \
      uint64_t res_lo = 0; \
      uint64_t res_hi = 0; \
      for (unsigned b = 0; b < bits - 8; b++) \
         res_hi |= lut[input[b]] << (bits - 9 - b); \
      for (unsigned b = bits - 8; b < bits; b++) \
         res_lo |= lut[input[b]] << (bits - 1 - b); \
\
      for (unsigned s = 0; s < 8; s++) \
      { \
         uint16_t v = (res_lo & 0xff) | ((res_hi & 0xff) << 8); \
         res_lo >>= 8; \
         res_hi >>= 8; \
         output[s] = ((int16_t)(v << (16 - bits))) >> (16 - bits); \
      } \
   } \
   return input; \
}

#define UNPACK_4_8(bits) \
const uint8_t *linelet_unpack_line4_##bits##_c(int16_t *output, const uint8_t *input, const uint64_t *lut) \
{ \
   uint32_t res = 0; \
   for (unsigned b = 0; b < bits; b++) \
      res |= lut[(input[b >> 1] >> (4 * (b & 1))) & 0x0f] << (bits - 1 - b); \
\
   for (unsigned s = 0; s < 4; s++) \
   { \
      uint8_t v = res; \
      res >>= 8; \
      output[s] = ((int16_t)(v << (16 - bits))) >> (16 - bits); \
   } \
   return input + ((bits + 1) >> 1); \
}

#define UNPACK_4_16(bits) \
const uint8_t *linelet_unpack_line4_##bits##_c(int16_t *output, const uint8_t *input, const uint64_t *lut) \
{ \
   uint32_t res_lo = 0; \
   uint32_t res_hi = 0; \
   for (unsigned b = 0; b < bits - 8; b++) \
      res_hi |= lut[(input[b >> 1] >> (4 * (b & 1))) & 0xf] << (bits - 9 - b); \
   for (unsigned b = bits - 8; b < bits; b++) \
      res_lo |= lut[(input[b >> 1] >> (4 * (b & 1))) & 0xf] << (bits - 1 - b); \
\
   for (unsigned s = 0; s < 4; s++) \
   { \
      uint16_t v = (res_lo & 0xff) | ((res_hi & 0xff) << 8); \
      res_lo >>= 8; \
      res_hi >>= 8; \
      output[s] = ((int16_t)(v << (16 - bits))) >> (16 - bits); \
   } \
   return input + ((bits + 1) >> 1); \
}

// Templates please.
PACK_16(1)
PACK_16(2)
PACK_16(3)
PACK_16(4)
PACK_16(5)
PACK_16(6)
PACK_16(7)
PACK_16(8)
PACK_16(9)
PACK_16(10)
PACK_16(11)
PACK_16(12)
PACK_16(13)
PACK_16(14)
PACK_16(15)
PACK_16(16)

PACK_8(1)
PACK_8(2)
PACK_8(3)
PACK_8(4)
PACK_8(5)
PACK_8(6)
PACK_8(7)
PACK_8(8)
PACK_8(9)
PACK_8(10)
PACK_8(11)
PACK_8(12)
PACK_8(13)
PACK_8(14)
PACK_8(15)
PACK_8(16)

PACK_4(1)
PACK_4(2)
PACK_4(3)
PACK_4(4)
PACK_4(5)
PACK_4(6)
PACK_4(7)
PACK_4(8)
PACK_4(9)
PACK_4(10)
PACK_4(11)
PACK_4(12)
PACK_4(13)
PACK_4(14)
PACK_4(15)
PACK_4(16)

UNPACK_8_8(1)
UNPACK_8_8(2)
UNPACK_8_8(3)
UNPACK_8_8(4)
UNPACK_8_8(5)
UNPACK_8_8(6)
UNPACK_8_8(7)
UNPACK_8_8(8)
UNPACK_8_16(9)
UNPACK_8_16(10)
UNPACK_8_16(11)
UNPACK_8_16(12)
UNPACK_8_16(13)
UNPACK_8_16(14)
UNPACK_8_16(15)
UNPACK_8_16(16)

UNPACK_4_8(1)
UNPACK_4_8(2)
UNPACK_4_8(3)
UNPACK_4_8(4)
UNPACK_4_8(5)
UNPACK_4_8(6)
UNPACK_4_8(7)
UNPACK_4_8(8)
UNPACK_4_16(9)
UNPACK_4_16(10)
UNPACK_4_16(11)
UNPACK_4_16(12)
UNPACK_4_16(13)
UNPACK_4_16(14)
UNPACK_4_16(15)
UNPACK_4_16(16)

