#include "legall.h"
#include "timer.h"
#include "simd.h"

void linelet_legall_vert_dwt_forward_top(
      int16_t * restrict lp0,
      int16_t * restrict hp0,
      int16_t * restrict lp1,
      unsigned samples)
{
   TIMER_COUNTER(legall_forward_vert_top);
   TIMER_START_COUNT(legall_forward_vert_top, samples);

   linelet_legall_vert_line_forward_top_t simd = linelet_simd.legall_vert_line_forward_top;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(lp0, hp0, lp1, x);
   }

   for (; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];

      h0 -= (l0 + l1) >> 1;
      l0 += ((h0 << 1) + 2) >> 2;

      lp0[x] = l0;
      hp0[x] = h0;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_forward_vert_top);
}

void linelet_legall_vert_dwt_forward(
      int16_t * restrict hp0,
      int16_t * restrict lp1,
      int16_t * restrict hp1,
      int16_t * restrict lp2,
      unsigned samples)
{
   TIMER_COUNTER(legall_forward_vert);
   TIMER_START_COUNT(legall_forward_vert, samples);

   linelet_legall_vert_line_forward_t simd = linelet_simd.legall_vert_line_forward;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(hp0, lp1, hp1, lp2, x);
   }

   for (; x < samples; x++)
   {
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];
      int16_t l2 = lp2[x];

      h1 -= (l1 + l2) >> 1;
      l1 += (h0 + h1 + 2) >> 2;

      lp1[x] = l1;
      hp1[x] = h1;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_forward_vert);
}

void linelet_legall_vert_dwt_forward_bottom(
      int16_t * restrict hp0,
      int16_t * restrict lp1,
      int16_t * restrict hp1,
      unsigned samples)
{
   TIMER_COUNTER(legall_forward_vert_bottom);
   TIMER_START_COUNT(legall_forward_vert_bottom, samples);

   linelet_legall_vert_line_forward_bottom_t simd = linelet_simd.legall_vert_line_forward_bottom;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(hp0, lp1, hp1, x);
   }

   for (; x < samples; x++)
   {
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];

      h1 -= l1;
      l1 += (h0 + h1 + 2) >> 2;

      lp1[x] = l1;
      hp1[x] = h1;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_forward_vert_bottom);
}

void linelet_legall_vert_dwt_inverse_top(
      int16_t * restrict lp0,
      int16_t * restrict hp0,
      unsigned samples)
{
   TIMER_COUNTER(legall_inverse_vert_top);
   TIMER_START_COUNT(legall_inverse_vert_top, samples);

   linelet_legall_vert_line_inverse_top_t simd = linelet_simd.legall_vert_line_inverse_top;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(lp0, hp0, x);
   }

   for (; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];

      l0 -= (h0 + 1) >> 1;
      lp0[x] = l0;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_inverse_vert_top);
}

void linelet_legall_vert_dwt_inverse(
      int16_t * restrict lp0,
      int16_t * restrict hp0,
      int16_t * restrict lp1,
      int16_t * restrict hp1,
      unsigned samples)
{
   TIMER_COUNTER(legall_inverse_vert);
   TIMER_START_COUNT(legall_inverse_vert, samples);

   linelet_legall_vert_line_inverse_t simd = linelet_simd.legall_vert_line_inverse;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(lp0, hp0, lp1, hp1, x);
   }

   for (; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];
      int16_t l1 = lp1[x];
      int16_t h1 = hp1[x];

      l1 -= (h0 + h1 + 2) >> 2;
      h0 += (l0 + l1) >> 1;

      hp0[x] = h0;
      lp1[x] = l1;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_inverse_vert);
}

void linelet_legall_vert_dwt_inverse_bottom(
      int16_t * restrict lp0,
      int16_t * restrict hp0,
      unsigned samples)
{
   TIMER_COUNTER(legall_inverse_vert_bottom);
   TIMER_START_COUNT(legall_inverse_vert_bottom, samples);

   linelet_legall_vert_line_inverse_bottom_t simd = linelet_simd.legall_vert_line_inverse_bottom;

   unsigned x = 0;
   if (samples >= 8)
   {
      x = samples & ~7;
      simd(lp0, hp0, x);
   }

   for (; x < samples; x++)
   {
      int16_t l0 = lp0[x];
      int16_t h0 = hp0[x];

      h0 += l0;
      hp0[x] = h0;
   }
   linelet_simd.emms();
   TIMER_STOP(legall_inverse_vert_bottom);
}

void linelet_legall_dwt_forward(int16_t * restrict lp, int16_t * restrict hp, const int16_t * restrict in, unsigned samples)
{
   TIMER_COUNTER(legall_forward);
   TIMER_START_COUNT(legall_forward, samples);

   unsigned x = 2;
   unsigned target = 1;

   if (samples >= 32)
   {
      unsigned asm_samples = (samples - 16) & ~15;
      linelet_simd.legall_line_forward(lp, hp, in, asm_samples);
      x = asm_samples;
      target = asm_samples >> 1;
   }

   // Fixups.
   hp[0] = in[1] - ((in[0] + in[2]) >> 1);
   lp[0] = in[0] + (((hp[0] << 1) + 2) >> 2);

   for (; x < samples - 2; x += 2, target++)
   {
      hp[target] = in[x + 1] - ((in[x] + in[x + 2]) >> 1);
      lp[target] = in[x] + ((hp[target - 1] + hp[target] + 2) >> 2);
   }

   unsigned last_target = (samples - 1) >> 1;
   hp[last_target] = in[samples - 1] - in[samples - 2];
   lp[last_target] = in[samples - 2] + ((hp[last_target - 1] + hp[last_target] + 2) >> 2);

   linelet_simd.emms();
   TIMER_STOP(legall_forward);
}

void linelet_legall_dwt_inverse(int16_t * restrict out, const int16_t * restrict lp, const int16_t * restrict hp, unsigned samples)
{
   TIMER_COUNTER(legall_inverse);
   TIMER_START_COUNT(legall_inverse, samples);

   out[0] = lp[0] - (((hp[0] << 1) + 2) >> 2);

   unsigned x = 2;
   unsigned target = 1;

   if (samples >= 32)
   {
      unsigned asm_samples = (samples - 16) & ~15;
      linelet_simd.legall_line_inverse(out + 2, lp + 1, hp + 1, asm_samples);
      x = asm_samples + 2;
      target = x >> 1;
   }

   for (; x + 2 <= samples; x += 2, target++)
   {
      out[x + 0] = lp[target] - ((hp[target] + hp[target - 1] + 2) >> 2);
      out[x - 1] = hp[target - 1] + ((out[x - 2] + out[x]) >> 1);
   }

   unsigned last_x = samples - 2;
   unsigned last_target = last_x >> 1;
   out[last_x + 1] = hp[last_target] + out[last_x];

   linelet_simd.emms();
   TIMER_STOP(legall_inverse);
}

