#include "linelet.h"
#include "memory.h"
#include "simd.h"
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <pthread.h>

void linelet_common_set_component_levels(unsigned *levels, unsigned lines, unsigned ll_levels)
{
   levels[0] = ll_levels;
   // Cater to the vertical lifting scheme.
   // LUT works for both 2, 4, 8 and 16 lines.
   levels[ 1] = 1;
   levels[ 2] = 2;
   levels[ 3] = 1;
   levels[ 4] = 3;
   levels[ 5] = 1;
   levels[ 6] = 2;
   levels[ 7] = 1;
#if LINELET_MAX_LINES >= 16
   levels[ 8] = 4;
   levels[ 9] = 1;
   levels[10] = 2;
   levels[11] = 1;
   levels[12] = 3;
   levels[13] = 1;
   levels[14] = 2;
   levels[15] = 1;
#endif
}

bool linelet_common_allocate_buffers(struct linelet_wavelet_buffer *buffers,
      unsigned full_samples, unsigned lines,
      const struct linelet_component *components, unsigned num_components)
{
   unsigned levels[LINELET_MAX_LINES];
   for (unsigned c = 0; c < num_components; c++)
   {
      linelet_common_set_component_levels(levels, lines, components[c].decomposition_levels);

      for (unsigned line = 0; line < lines; line++)
      {
         for (unsigned l = 0; l < levels[line]; l++)
         {
            unsigned samples = full_samples >> components[c].decimation;
            samples >>= l + 1;
            buffers[c].lp[line][l] = linelet_calloc(samples, sizeof(int16_t));
            buffers[c].hp[line][l] = linelet_calloc(samples, sizeof(int16_t));
            if (!buffers[c].lp[line][l] || !buffers[c].hp[line][l])
               return false;
         }
      }
   }

   return true;
}

void linelet_common_free_buffers(struct linelet_wavelet_buffer *buffers, unsigned num_components)
{
   for (unsigned c = 0; c < num_components; c++)
   {
      for (unsigned line = 0; line < LINELET_MAX_LINES; line++)
      {
         for (unsigned l = 0; l < LINELET_MAX_DECOMP_LEVELS; l++)
         {
            linelet_free(buffers[c].lp[line][l]);
            linelet_free(buffers[c].hp[line][l]);
         }
      }
   }

   memset(buffers, 0, sizeof(*buffers) * num_components);
}

static void simd_init(void)
{
   memset(&linelet_simd, 0, sizeof(linelet_simd));
   linelet_simd_init_c();
#ifdef LINELET_SIMD_X86
   linelet_simd_init_x86();
#endif
}

void linelet_init(void)
{
   static pthread_once_t init_once = PTHREAD_ONCE_INIT;
   pthread_once(&init_once, simd_init);
}

