#include "linelet_format.h"
#include <string.h>

enum linelet_format_format linelet_format_parse(const char *arg, const char *matrix)
{
   if (!arg)
      return LINELET_FMT_UNKNOWN;

   bool bt601 = matrix && !strcmp(matrix, "bt601");

   if (!strcmp(arg, "yuv444p"))
      return bt601 ? LINELET_FMT_YCBCR8_BT601_444 : LINELET_FMT_YCBCR8_BT709_444;
   else if (!strcmp(arg, "yuv422p"))
      return bt601 ? LINELET_FMT_YCBCR8_BT601_422 : LINELET_FMT_YCBCR8_BT709_422;
   else if (!strcmp(arg, "yuv444p10"))
      return bt601 ? LINELET_FMT_YCBCR10_BT601_444 : LINELET_FMT_YCBCR10_BT709_444;
   else if (!strcmp(arg, "yuv422p10"))
      return bt601 ? LINELET_FMT_YCBCR10_BT601_422 : LINELET_FMT_YCBCR10_BT709_422;
   else if (!strcmp(arg, "yuv444p12"))
      return bt601 ? LINELET_FMT_YCBCR12_BT601_444 : LINELET_FMT_YCBCR12_BT709_444;
   else if (!strcmp(arg, "yuv422p12"))
      return bt601 ? LINELET_FMT_YCBCR12_BT601_422 : LINELET_FMT_YCBCR12_BT709_422;
   else if (!strcmp(arg, "gbrp"))
      return LINELET_FMT_GBRP8;
   else if (!strcmp(arg, "gbrp10"))
      return LINELET_FMT_GBRP10;
   else
      return LINELET_FMT_UNKNOWN;
}

unsigned linelet_format_to_subsample(enum linelet_format_format fmt)
{
   switch (fmt)
   {
      case LINELET_FMT_YCBCR12_BT709_422:
      case LINELET_FMT_YCBCR10_BT709_422:
      case LINELET_FMT_YCBCR8_BT709_422:

      case LINELET_FMT_YCBCR12_BT601_422:
      case LINELET_FMT_YCBCR10_BT601_422:
      case LINELET_FMT_YCBCR8_BT601_422:

      case LINELET_FMT_YCBCR10_RCT_422:
      case LINELET_FMT_YCBCR8_RCT_422:
         return 1;

      default:
         return 0;
   }
}

unsigned linelet_format_to_bpp(enum linelet_format_format fmt)
{
   switch (fmt)
   {
      case LINELET_FMT_YCBCR8_BT601_422:
      case LINELET_FMT_YCBCR8_BT601_444:
      case LINELET_FMT_YCBCR8_BT709_422:
      case LINELET_FMT_YCBCR8_BT709_444:
      case LINELET_FMT_GBRP8:
         return 1;

      case LINELET_FMT_YCBCR10_BT709_422:
      case LINELET_FMT_YCBCR10_BT709_444:
      case LINELET_FMT_YCBCR10_BT601_422:
      case LINELET_FMT_YCBCR10_BT601_444:
      case LINELET_FMT_YCBCR12_BT709_422:
      case LINELET_FMT_YCBCR12_BT709_444:
      case LINELET_FMT_YCBCR12_BT601_422:
      case LINELET_FMT_YCBCR12_BT601_444:

      case LINELET_FMT_YCBCR8_RCT_444:
      case LINELET_FMT_YCBCR8_RCT_422:
      case LINELET_FMT_YCBCR10_RCT_444:
      case LINELET_FMT_YCBCR10_RCT_422:
      case LINELET_FMT_GBRP10:
         return 2;

      default:
         return 0;
   }
}

enum linelet_yuv_transform linelet_format_to_transform(enum linelet_format_format fmt)
{
   switch (fmt)
   {
      case LINELET_FMT_YCBCR8_RCT_444:
      case LINELET_FMT_YCBCR8_RCT_422:
      case LINELET_FMT_YCBCR10_RCT_444:
      case LINELET_FMT_YCBCR10_RCT_422:
      case LINELET_FMT_X2B10G10R10:
      case LINELET_FMT_X8R8G8B8:
      case LINELET_FMT_GBRP10:
      case LINELET_FMT_GBRP8:
         return LINELET_YUV_RCT;

      case LINELET_FMT_YCBCR8_BT709_444:
      case LINELET_FMT_YCBCR8_BT709_422:
      case LINELET_FMT_YCBCR10_BT709_444:
      case LINELET_FMT_YCBCR10_BT709_422:
      case LINELET_FMT_YCBCR12_BT709_444:
      case LINELET_FMT_YCBCR12_BT709_422:
         return LINELET_YUV_BT709;

      case LINELET_FMT_YCBCR8_BT601_444:
      case LINELET_FMT_YCBCR8_BT601_422:
      case LINELET_FMT_YCBCR10_BT601_444:
      case LINELET_FMT_YCBCR10_BT601_422:
      case LINELET_FMT_YCBCR12_BT601_444:
      case LINELET_FMT_YCBCR12_BT601_422:
         return LINELET_YUV_BT601;

      default:
         return LINELET_YUV_UNKNOWN;
   }
}

unsigned linelet_format_to_components(enum linelet_format_format fmt)
{
   return (fmt == LINELET_FMT_X2B10G10R10 || fmt == LINELET_FMT_X8R8G8B8) ? 1 : 3;
}

unsigned linelet_format_to_bits(enum linelet_format_format fmt)
{
   switch (fmt)
   {
      case LINELET_FMT_YCBCR8_RCT_444:
      case LINELET_FMT_YCBCR8_RCT_422:
      case LINELET_FMT_YCBCR8_BT709_444:
      case LINELET_FMT_YCBCR8_BT709_422:
      case LINELET_FMT_YCBCR8_BT601_444:
      case LINELET_FMT_YCBCR8_BT601_422:
      case LINELET_FMT_X8R8G8B8:
      case LINELET_FMT_GBRP8:
         return 8;

      case LINELET_FMT_YCBCR10_RCT_444:
      case LINELET_FMT_YCBCR10_RCT_422:
      case LINELET_FMT_YCBCR10_BT709_444:
      case LINELET_FMT_YCBCR10_BT709_422:
      case LINELET_FMT_YCBCR10_BT601_444:
      case LINELET_FMT_YCBCR10_BT601_422:
      case LINELET_FMT_X2B10G10R10:
      case LINELET_FMT_GBRP10:
         return 10;

      case LINELET_FMT_YCBCR12_BT709_444:
      case LINELET_FMT_YCBCR12_BT709_422:
      case LINELET_FMT_YCBCR12_BT601_444:
      case LINELET_FMT_YCBCR12_BT601_422:
         return 12;

      default:
         return 0;
   }
}

