#include "range.h"
#include "timer.h"
#include "rate_allocator.h"
#include "simd.h"

void linelet_ranges_min_max(struct linelet_precinct_range_level *ranges, const int16_t *src, unsigned precincts, unsigned step)
{
   if (step >= 8)
   {
      TIMER_COUNTER(ranges_min_max_sse);
      TIMER_START_COUNT(ranges_min_max_sse, precincts * step);
      linelet_simd.range_min_max_line_8(ranges, src, precincts, step);
      TIMER_STOP(ranges_min_max_sse);
   }
   else if (step == 4)
   {
      TIMER_COUNTER(ranges_min_max_sse_4);
      TIMER_START_COUNT(ranges_min_max_sse_4, precincts * step);

      unsigned p = precincts & ~1;
      linelet_simd.range_min_max_line_4(ranges, src, p);
      src += p * 4;
      ranges += p;

      // Cleanup.
      for (; p < precincts; p++, src += 4, ranges++)
      {
         int16_t lo = 0;
         int16_t hi = 0;
         for (unsigned x = 0; x < 4; x++)
         {
            if (src[x] < lo)
               lo = src[x];
            else if (src[x] > hi)
               hi = src[x];
         }

         ranges->lo = lo;
         ranges->hi = hi;
      }

      TIMER_STOP(ranges_min_max_sse_4);
   }
   else
   {
      TIMER_COUNTER(ranges_min_max_c);
      TIMER_START_COUNT(ranges_min_max_c, precincts * step);
      for (unsigned p = 0; p < precincts; p++, src += step, ranges++)
      {
         int16_t lo = 0;
         int16_t hi = 0;
         for (unsigned x = 0; x < step; x++)
         {
            if (src[x] < lo)
               lo = src[x];
            else if (src[x] > hi)
               hi = src[x];
         }

         ranges->lo = lo;
         ranges->hi = hi;
      }
      TIMER_STOP(ranges_min_max_c);
   }

   linelet_simd.emms();
}

