%include "inc/x86inc.asm"
%include "inc/x86util.asm"

; s1 = s0 - s1
; s0 -= s1 >> 1
%macro LIFT1 5 ; s0, s1, mem0, mem1, tmp
   mova %1, %3
   mova %2, %1
   psubw %2, %4

   mova %5, %2
   psraw %5, 1
   psubw %1, %5
%endmacro

; s0 += s1 >> 1
; s1 = s0 - s1
%macro LIFT2 3 ; s0, s1, tmp
   mova %3, %2
   psraw %2, 1
   paddw %1, %2

   mova %2, %1
   psubw %2, %3
%endmacro

%macro PREFILTER2 0
; void prefilter_line2(int16_t *l0, int16_t *l1, unsigned samples)
cglobal prefilter_line2, 3, 3, 3
   sub r1, r0

.loop:
   LIFT1 m0, m1, [r0], [r0 + r1], m2

   ; Can combine V and post-lift here, saves a cycle I suppose.

   ; s0 += (s1 << 1) >> 1
   paddw m0, m1
   ; s1 <<= 1
   psllw m1, 1

   ; s1 = s0 - s1
   mova m2, m0
   psubw m2, m1

   mova [r0], m0
   mova [r0 + r1], m2

   add r0, mmsize
   sub r2d, mmsize / 2
   jg .loop
   
   RET
%endmacro

%macro PREFILTER4 0
; void prefilter_line4(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, unsigned samples)
cglobal prefilter_line4, 5, 5, 4
   sub r3, r0
   sub r2, r0
   sub r1, r0

.loop:
   ; (s0 - s3) >> 1
   mova m3, [r0]
   psubw m3, [r0 + r3]
   psraw m3, 1

   LIFT1 m1, m2, [r0 + r1], [r0 + r2], m0

   ; s2 <<= 1, s2 += (s0 - s3) >> 1
   psllw m2, 1
   paddw m2, m3

   LIFT2 m1, m2, m0

   mova [r0 + r1], m1
   mova [r0 + r2], m2

   add r0, mmsize
   sub r4d, mmsize / 2
   jg .loop
   
   RET
%endmacro

%macro ADD_SHIFTED 4 ; dst, src, shift, tmp
   mova %4, %2
   psraw %4, %3
   paddw %1, %4
%endmacro

%macro SUB_SHIFTED 4 ; dst, src, shift, tmp
   mova %4, %2
   psraw %4, %3
   psubw %1, %4
%endmacro

%macro PREFILTER8 0
; void prefilter_line8(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3,
;                      int16_t *l4, int16_t *l5, int16_t *l6, int16_t *l7, unsigned samples)
cglobal prefilter_line8, 9, 9, 8
   sub r7, r0
   sub r6, r0
   sub r5, r0
   sub r4, r0
   sub r3, r0
   sub r2, r0
   sub r1, r0

.loop:
   ; (s0 - s7) >> 3
   mova m7, [r0]
   psubw m7, [r0 + r7]
   psraw m7, 3

   LIFT1 m1, m6, [r0 + r1], [r0 + r6], m0
   LIFT1 m2, m5, [r0 + r2], [r0 + r5], m0
   LIFT1 m3, m4, [r0 + r3], [r0 + r4], m0

   psllw m4, 1
   paddw m6, m7
   ADD_SHIFTED m5, m6, 2, m0
   ADD_SHIFTED m4, m5, 1, m0

   LIFT2 m3, m4, m0
   LIFT2 m2, m5, m0
   LIFT2 m1, m6, m0

   mova [r0 + r1], m1
   mova [r0 + r2], m2
   mova [r0 + r3], m3
   mova [r0 + r4], m4
   mova [r0 + r5], m5
   mova [r0 + r6], m6

   add r0, mmsize
   sub r8d, mmsize / 2
   jg .loop

   RET
%endmacro

%macro POSTFILTER2 0
; void postfilter_line2(int16_t *l0, int16_t *l1, unsigned samples)
cglobal postfilter_line2, 3, 3, 3
   sub r1, r0

.loop:
   ; Can combine V and first lift somewhat.
   ; s1 = s0 - s1
   mova m0, [r0]
   mova m1, m0
   psubw m1, [r0 + r1]

   ; s1 >>= 1, s0 -= s1
   psraw m1, 1
   psubw m0, m1

   LIFT2 m0, m1, m2

   mova [r0], m0
   mova [r0 + r1], m1

   add r0, mmsize
   sub r2d, mmsize / 2
   jg .loop
   
   RET
%endmacro

%macro POSTFILTER4 0
; void postfilter_line4(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3, unsigned samples)
cglobal postfilter_line4, 5, 5, 4
   sub r3, r0
   sub r2, r0
   sub r1, r0

.loop:
   ; (s0 - s3) >> 1
   mova m3, [r0]
   psubw m3, [r0 + r3]
   psraw m3, 1

   LIFT1 m1, m2, [r0 + r1], [r0 + r2], m0

   ; s2 -= (s0 - s3) >> 1, s2 >>= 1
   psubw m2, m3
   psraw m2, 1

   LIFT2 m1, m2, m0

   mova [r0 + r1], m1
   mova [r0 + r2], m2

   add r0, mmsize
   sub r4d, mmsize / 2
   jg .loop
   
   RET
%endmacro

%macro POSTFILTER8 0
; void postfilter_line8(int16_t *l0, int16_t *l1, int16_t *l2, int16_t *l3,
;                      int16_t *l4, int16_t *l5, int16_t *l6, int16_t *l7, unsigned samples)
cglobal postfilter_line8, 9, 9, 8
   sub r7, r0
   sub r6, r0
   sub r5, r0
   sub r4, r0
   sub r3, r0
   sub r2, r0
   sub r1, r0

.loop:
   ; (s0 - s7) >> 3
   mova m7, [r0]
   psubw m7, [r0 + r7]
   psraw m7, 3

   LIFT1 m1, m6, [r0 + r1], [r0 + r6], m0
   LIFT1 m2, m5, [r0 + r2], [r0 + r5], m0
   LIFT1 m3, m4, [r0 + r3], [r0 + r4], m0

   SUB_SHIFTED m4, m5, 1, m0
   SUB_SHIFTED m5, m6, 2, m0
   psubw m6, m7
   psraw m4, 1

   LIFT2 m3, m4, m0
   LIFT2 m2, m5, m0
   LIFT2 m1, m6, m0

   mova [r0 + r1], m1
   mova [r0 + r2], m2
   mova [r0 + r3], m3
   mova [r0 + r4], m4
   mova [r0 + r5], m5
   mova [r0 + r6], m6

   add r0, mmsize
   sub r8d, mmsize / 2
   jg .loop

   RET
%endmacro

INIT_XMM sse2
PREFILTER2
PREFILTER4
POSTFILTER2
POSTFILTER4
%if ARCH_X86_64 == 1 ; Using more than 6 GPR. Don't bother with hacky code to get it working on 32-bit.
PREFILTER8
POSTFILTER8
%endif
INIT_MMX mmx2
PREFILTER2
PREFILTER4
POSTFILTER2
POSTFILTER4

