#!/usr/bin/env python

import sys
import os
import subprocess
import math

rates = [0.03, 0.05, 0.07, 0.10, 0.12, 0.15, 0.18, 0.20, 0.25, 0.30, 0.35, 0.40, 0.50]

def measure(f, lines, filt, rate, method):
    target = '{}_{}_{}_{}_{}.txt'.format(os.path.splitext(os.path.basename(f))[0], int(rate * 100), lines, filt, method)
    cmd = 'linelet encode -i {} -o - --size 1920x1080 --precinct 64 --pix_fmt yuv422p10 -S {} --filter {} -r {} -t 2 -q | linelet decode -i - -o - | linelet {}_video - {} --size 1920x1080 --pix_fmt yuv422p10 > {}'.format(f, lines, filt, rate, method, f, target)
    subprocess.check_call(cmd, shell = True)

def run_test(f):
    for rate in rates:
        for l in [1, 2, 4, 8, 16]:
            filt = 3 if l == 16 else int(math.log2(l))
            measure(f, l, filt, rate, 'psnr')
            measure(f, l, filt, rate, 'ssim')
            measure(f, l, 0, rate, 'psnr')
            measure(f, l, 0, rate, 'ssim')

if __name__ == '__main__':
    for f in sys.argv[1:]:
        run_test(f)
