#include "tests.h"
#include "memory.h"

int format_test_422(void)
{
   linelet_format_encoder_t *enc = linelet_format_encoder_new(&(struct linelet_format_encoder_options) {
            .format = LINELET_FMT_YCBCR8_BT709_422,
            .width = 32,
            .height = 2,
            .precinct_samples = 16,
            .decomposition_levels = 2,
            .slice_height = 1,
            .threads = 1,
         });

   if (!enc)
      return -1;

   static const uint8_t luma[64] ALIGNED_STACK = {
      0x11, 0x22, 0x44, 0x66, 0x11, 0x05, 0x09, 0xff,
      0xaa, 0x22, 0xaa, 0x66, 0x11, 0xaa, 0x09, 0xaa,
      0xaa, 0xbb, 0xaa, 0x66, 0xbb, 0xaa, 0x09, 0xaa,
      0xaa, 0x22, 0xaa, 0x66, 0x11, 0xaa, 0x09, 0xaa,
      0x11, 0x22, 0x44, 0x66, 0x11, 0x05, 0x09, 0xff,
      0xaa, 0x22, 0xaa, 0x66, 0x11, 0xaa, 0x09, 0xaa,
      0xaa, 0xbb, 0xaa, 0x66, 0xbb, 0xaa, 0x09, 0xaa,
      0x11, 0x22, 0x44, 0x66, 0x11, 0x05, 0x09, 0xff,
   };

   static const uint8_t cb[32] ALIGNED_STACK = {
      0, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb,
      1, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb,
      2, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb,
      3, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb, 0xcb,
   };

   static const uint8_t cr[32] ALIGNED_STACK = {
      4, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf,
      5, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf,
      6, 0xcf, 0xcf, 0xcf, 0xff, 0xcf, 0xcf, 0xcf,
      7, 0xcf, 0xcf, 0xcf, 0xff, 0xcf, 0xcf, 0xcf,
   };

   uint8_t encoder_buffer[1024] ALIGNED_STACK;
   linelet_format_encoder_process_frame(enc, (const void*[]) { luma, cb, cr },
         (const size_t[]) { 32, 16, 16 }, encoder_buffer, sizeof(encoder_buffer));

   linelet_format_encoder_free(enc);

   struct linelet_format_decoder_frame_params params;
   if (linelet_format_decoder_parse_header(&params, encoder_buffer, sizeof(encoder_buffer)))
   {
      if (params.width != 32 || params.height != 2 || params.format != LINELET_FMT_YCBCR8_BT709_422)
         return -2;
   }
   else
      return -3;

   linelet_format_decoder_t *dec = linelet_format_decoder_new(&(struct linelet_format_decoder_options) {
            .format = LINELET_FMT_YCBCR8_BT709_422,
            .threads = 1,
         });
   if (!dec)
      return -4;

   uint8_t out_luma[64] ALIGNED_STACK = {0}, out_cb[32] ALIGNED_STACK = {0}, out_cr[32] ALIGNED_STACK = {0};
   if (!linelet_format_decoder_process_frame(dec,
            encoder_buffer, sizeof(encoder_buffer),
            (void *[]) { out_luma, out_cb, out_cr },
            (const size_t[]) { 32, 16, 16 }))
      return -5;

   if (memcmp(out_luma, luma, sizeof(luma)))
      return -10;
   if (memcmp(out_cb, cb, sizeof(cb)))
      return -11;
   if (memcmp(out_cr, cr, sizeof(cr)))
      return -12;

   linelet_format_decoder_free(dec);
   return 0;
}

int format_test(void)
{
   linelet_format_encoder_t *enc = linelet_format_encoder_new(&(struct linelet_format_encoder_options) {
            .format = LINELET_FMT_X2B10G10R10,
            .width = 64,
            .height = 64,
            .precinct_samples = 16,
            .decomposition_levels = 2,
            .slice_height = 4,
            .threads = 4,
         });

   if (!enc)
      return -1;

   uint32_t rgb_data[64 * 64] ALIGNED_STACK;
   uint32_t ref_rgb_data[64 * 64] ALIGNED_STACK;
   for (unsigned y = 0; y < 64; y++)
   {
      for (unsigned x = 0; x < 64; x++)
      {
         uint32_t r = (x + y) << 3;
         uint32_t g = (x ^ y) << 4;
         uint32_t b = (x & y) << 4;
         rgb_data[(y * 64 + x)] = (0x3u << 30) | (b << 20) | (g << 10) | (r << 0);
      }
   }

   memcpy(ref_rgb_data, rgb_data, sizeof(rgb_data));

   uint8_t encoder_buffer[3 * 64 * 64 * 12] ALIGNED_STACK = {0};
   linelet_format_encoder_process_frame(enc, (const void *[]) { rgb_data }, (const size_t[]) { 64 * sizeof(uint32_t) },
         encoder_buffer, sizeof(encoder_buffer));

   linelet_format_encoder_free(enc);

   struct linelet_format_decoder_frame_params params;
   if (linelet_format_decoder_parse_header(&params, encoder_buffer, sizeof(encoder_buffer)))
   {
      if (params.width != 64 || params.height != 64 || params.format != LINELET_FMT_YCBCR10_RCT_444)
         return -2;
   }
   else
      return -3;

   linelet_format_decoder_t *dec = linelet_format_decoder_new(&(struct linelet_format_decoder_options) {
            .format = LINELET_FMT_X2B10G10R10,
            .threads = 4,
         });
   if (!dec)
      return -4;

   uint32_t target_rgb[64 * 64] ALIGNED_STACK;
   if (!linelet_format_decoder_process_frame(dec,
            encoder_buffer, sizeof(encoder_buffer),
            (void *[]) { target_rgb }, (const size_t[]) { 64 * sizeof(uint32_t) }))
      return -5;

   for (unsigned i = 0; i < 64 * 64; i++)
      target_rgb[i] |= 0xc0000000u;
   for (unsigned i = 0; i < 64; i++)
      if (memcmp(target_rgb + 64 * i, ref_rgb_data + 64 * i, 64 * sizeof(uint32_t)))
         return -10 - (int)i;

   linelet_format_decoder_free(dec);
   return 0;
}

