#include "tests.h"
#include "timer.h"
#include "linelet.h"
#include "log.h"
#include <stddef.h>
#include <stdio.h>

struct testcase
{
   int (*func)(void);
   const char *desc;
};

static const struct testcase cases[] = {
   { pack_unary, "Pack/Unpack unary" },
   { pack_u8, "Pack/Unpack u8" },
   { pack_s16, "Pack/Unpack s16" },
   { quantizer_test, "Quantizer" },
   { format_test, "Format encode/decode" },
   { format_test_422, "Format encode/decode (4:2:2)" },
   { convert_test, "Convert test" },
   { legall_test, "Legall test" },
   { haar_test, "Haar test" },
   { encoder_test, "Encoder/Decoder" },
   { encoder_range_test, "Encoder/Decoder Range" },
   { rate_allocator_test, "Rate allocator" },
   { prepostfilter_test, "Pre/Post filter" },
   { NULL, NULL },
};

void tests_run_all(void)
{
   linelet_init();

   unsigned succeeded = 0;
   unsigned failed = 0;
   for (size_t i = 0; cases[i].func; i++)
   {
      linelet_log_printf(LINELET_LOG_INFO, "Running test: %s\n", cases[i].desc);
      int ret = cases[i].func();

      if (ret)
         linelet_log_printf(LINELET_LOG_INFO, " ... FAILED! (%d)\n", ret);
      else
         linelet_log_printf(LINELET_LOG_INFO, " ... OK\n");

      if (!ret)
         succeeded++;
      else
         failed++;
   }

   linelet_log_printf(LINELET_LOG_INFO, "Succeeded tests: %u. Failed tests: %u.\n",
         succeeded, failed);
}

